<?php

##########################################################################
# This file contains some definable stuff, for configuration.
# Other definitions that has nothing to do with configuration can be found
# in general.inc.php
#

#
# PATHS AND FILENAMES
#

# Main path in the unix filesystem
define("UNIX_PATH",	"/var/www/htdocs/yala");
define("INCLUDE_PATH",	UNIX_PATH."/include");
define("IMAGES_PATH",	UNIX_PATH."/images");

# Path to place cache files in
define("CACHE_PATH",	"/tmp");

# URL PATHS
define("IMAGES_URLPATH",	"images");

# PHP filenames. usually shouldn't be changed
define("MAINFILE", "yala.php");
define("TREEFILE", "tree.php");

# Icon to display when no matching icon was found (inside IMAGES_PATH)
define("DEFAULT_ICON", "default.png");



#
# CACHE STUFF
#

define("NAME2OID_CACHEFILE", CACHE_PATH."/yala_name2oid.cache");
define("OBJECTCLASSES_CACHEFILE", CACHE_PATH."/yala_objectclasses.cache");
# Number of *seconds* after which the cache expires
define("CACHE_EXPIRES", 60*5);



#
# LDAP default stuff
#

define("LDAP_SERVER",	"127.0.0.1");
define("LDAP_PORT",	"389");
define("LDAP_BASEDN",	"dc=held,dc=org,dc=il");
define("LDAP_BINDDN",	"cn=root,dc=held,dc=org,dc=il");
define("LDAP_VERSION",	"3"); # No good reason to use LDAPv2, unless an old PHP or LDAP Server


#
# Design
# 

# Colors
define("COLOR_BG1",	"#CFCFDF");
define("COLOR_BG2",	"#EEEEFF");
define("COLOR_FGTEXT",	"#222266");

# Size of text input
define("INPUT_TEXT_SIZE", 40);

# Web Stuff
define("ENABLE_JAVASCRIPT", 1); # Neat js features :)



#
# Entry Types
#
# entry_types are yala's way to describe an entry type (i.e. 'user')
# by several objectclasses which always build it.
# This is used both for yala to recognize such objects and give them a
# special icon, and both for allowing easy add of entry of that type.
#
# Note: objectclass name's are CASE SENSITIVE!
# Note: 'custom' is already a registered Entry Type, don't use it.
#
$entry_types["user"] 	= array("account", "posixAccount", "shadowAccount");
$entry_types["group"]	= array("posixGroup");
$entry_types["ou"]	= array("organizationalUnit");



#
# Attribute Descriptions
# 
# $attr_desc is an associative array (hash) containing a list of values 
# (key) and their description
# (value). It'll be available to the user in some ways, i.e. <ACRONYM> 
# HTML Tag.
#
# Note: In future release the description will be taken directly from the 
# schema (DESC line in the attributetype block)
#

# General
$attr_desc["cn"] = "Common Name";
$attr_desc["sn"] = "Surname";

# User objects
$attr_desc["uidNumber"]		= "User ID: Unique ID of a user, i.e. 532";
$attr_desc["gidNumber"]		= "Group ID: The Unique ID of a group, i.e. 532";
$attr_desc["gecos"]		= "User's full name (aka GECOS field), i.e. John Smith";

?>
